#include <curl/curl.h>
#include "urlget.h"
#include "download_backend.h"

typedef struct _DownloadBackendCurl DownloadBackendCurl;

struct _DownloadBackendCurl
{
	CURL*   curl;
	char    error_string[CURL_ERROR_SIZE + 1];
	gdouble last_completed;
	glong   resume_from;

	DownloadThread* thread;
};


static int curl_progress_func (DownloadBackendCurl* backend,
                               double dl_total,
                               double dl_now,
                               double ul_total,
                               double ul_now);

void download_backend_global_init ()
{
	curl_global_init (CURL_GLOBAL_ALL);
}

void download_backend_global_clean ()
{
	curl_global_cleanup ();
}

gpointer download_backend_new (gpointer data)
{
	DownloadBackendCurl* backend;

	backend = g_malloc (sizeof (DownloadBackendCurl));
	backend->curl = curl_easy_init ();
	backend->thread = data;

	return backend;
}

void download_backend_destroy (gpointer data)
{
	DownloadBackendCurl* backend = data;

	curl_easy_cleanup (backend->curl);
	g_free (backend);
}

gint download_backend_run (gpointer backend, DownloadThread* dt)
{
	DownloadBackendCurl* dbc = backend;
	GString* filename = dt->out_filename;
	FILE*    file;
	DownloadSetting* setting = DOWNLOAD_SETTING (dt->download_data);

	CURL*    curl = dbc->curl;
	char*    curl_error_string = dbc->error_string;
	CURLcode curl_code;
	long     curl_proxy_type;

	dbc->last_completed = 0.0;

	// check file
	if (dt->resumable) {
		file = fopen (filename->str, "ab");
		fseek (file, 0, SEEK_END);
		dbc->resume_from = ftell (file);
	} else {
		file = fopen (filename->str, "wb");
		dbc->resume_from = 0;
	}
	if (file==NULL) {
		download_thread_job_set_message (dt, _("File open error."));
		return DOWNLOAD_BACKEND_ERROR;
	}

	// set curl proxy
	if (PROXY_SETTING (setting)->type == PROXY_TYPE_NONE) {
		curl_easy_setopt (curl, CURLOPT_PROXYTYPE, CURLPROXY_HTTP);
		curl_easy_setopt (curl, CURLOPT_PROXY, NULL);
	}
	else {
		switch (PROXY_SETTING (setting)->type) {
		default:
		case PROXY_TYPE_HTTP:
			curl_proxy_type = CURLPROXY_HTTP;
			break;
		case PROXY_TYPE_SOCKS4:
			curl_proxy_type = CURLPROXY_SOCKS4;
			break;
		case PROXY_TYPE_SOCKS5:
			curl_proxy_type = CURLPROXY_SOCKS5;
			break;
		}
		curl_easy_setopt (curl, CURLOPT_PROXYTYPE, curl_proxy_type);
		curl_easy_setopt (curl, CURLOPT_PROXY,
		                  PROXY_SETTING (setting)->address);
		curl_easy_setopt (curl, CURLOPT_PROXYPORT,
		                  (long)PROXY_SETTING (setting)->port);
	}

	// set redirection
    if (setting->n_redirect) {
        curl_easy_setopt (curl, CURLOPT_FOLLOWLOCATION, 1);
        curl_easy_setopt (curl, CURLOPT_MAXREDIRS, setting->n_redirect);
    } else {
        curl_easy_setopt (curl, CURLOPT_FOLLOWLOCATION, 0);
    }

	// set download data
	curl_easy_setopt (curl, CURLOPT_URL, setting->url);
	curl_easy_setopt (curl, CURLOPT_FILE, file);
	curl_easy_setopt (curl, CURLOPT_NOPROGRESS, FALSE);
	curl_easy_setopt (curl, CURLOPT_PROGRESSFUNCTION, curl_progress_func);
	curl_easy_setopt (curl, CURLOPT_PROGRESSDATA, dbc);
	curl_easy_setopt (curl, CURLOPT_RESUME_FROM, dbc->resume_from);
	curl_easy_setopt (curl, CURLOPT_ERRORBUFFER, curl_error_string);

	// perform curl
	curl_code = curl_easy_perform (curl);
	fclose (file);

//	dt->time_state.
//	set_job_speed (NULL);

	if (curl_code == CURLE_OK) {
		Download* dld = dt->download_data;

		if (dld->completed + dbc->resume_from == dld->total + dbc->resume_from &&
		    dld->completed + dbc->resume_from != 0 )
		{
			return DOWNLOAD_BACKEND_OK;
		}
		else {
			unlink (filename->str);
			download_thread_job_set_message (dt, _("File length error. Try to increase redirection limit."));
			return DOWNLOAD_BACKEND_ERROR;
		}
	}

	printf ("Curl error %.2u: %s\n", curl_code, curl_error_string);
	download_thread_job_set_message (dt, curl_error_string);

	// check curl return code
	switch (curl_code) {
		// can resume
		case CURLE_PARTIAL_FILE:
		case CURLE_GOT_NOTHING:
			dt->resumable = TRUE;
			break;
		// can't resume
		case CURLE_HTTP_RANGE_ERROR:
			dt->resumable = FALSE;
			dbc->resume_from = 0;
			break;
		case CURLE_ABORTED_BY_CALLBACK:
			return DOWNLOAD_BACKEND_ABORT;
//		case CURLE_UNSUPPORTED_PROTOCOL:
//		case CURLE_FAILED_INIT:
//		case CURLE_URL_MALFORMAT:
//		case CURLE_URL_MALFORMAT_USER:
//		case CURLE_COULDNT_RESOLVE_PROXY:
//		case CURLE_COULDNT_RESOLVE_HOST:
//		case CURLE_COULDNT_CONNECT:
//		case CURLE_FTP_WEIRD_SERVER_REPLY:
//		case CURLE_FTP_ACCESS_DENIED:
//		case CURLE_FTP_USER_PASSWORD_INCORRECT:
		default:
			unlink (filename->str);
			return DOWNLOAD_BACKEND_ERROR;
	}

	return DOWNLOAD_BACKEND_CAN_RETRY;
}

static int curl_progress_func (DownloadBackendCurl* dbc,
                               double dl_total,
                               double dl_now,
                               double ul_total,
                               double ul_now)
{
	DownloadThread* dt = dbc->thread;
	Download* download_data = dt->download_data;
	gdouble timer = download_thread_get_timer (dt);
	gdouble speed;

	if (dl_now) {
		download_thread_job_set_info_valid (dt, TRUE);
		speed = (dl_now - dbc->last_completed) / timer;
		download_data->speed = (download_data->speed + speed) /2;
		download_data->completed = dl_now + dbc->resume_from;
		download_data->total     = dl_total + dbc->resume_from;
		dbc->last_completed = dl_now;
		// curl pass dl_total=0 sometimes.
		if (download_data->total < download_data->completed)
			download_data->total = download_data->completed;
	}

	// avoid div zero
	if (download_data->total)
		download_data->percent = (download_data->completed / download_data->total)*100;

	download_thread_job_redraw (dt);
	// return true when abort.
	return (dt->thread_state >= DOWNLOAD_THREAD_SKIP);
}


