#include <stdlib.h>
#include "download_store.h"
#include "download_thread.h"

void download_store_init (DownloadStore* ds)
{
	ds->list_store = gtk_list_store_new (DOWNLOAD_N_COL,
	                                     G_TYPE_POINTER,
	                                     G_TYPE_POINTER);

	ds->capacity_limit = 0;
	ds->historical = FALSE;
	g_static_rec_mutex_init (&ds->mutex);
}

void download_store_clear (DownloadStore* ds)
{
	// remove data
	download_store_erase_from_nth (ds, 0);
	g_object_unref (ds->list_store);

	g_static_rec_mutex_free (&ds->mutex);
}

void download_store_add_static (DownloadStore* ds, Download* dl)
{
	GtkTreeIter iter;

	download_store_lock (ds);

	if (ds->historical)
		gtk_list_store_prepend (ds->list_store, &iter);
	else
		gtk_list_store_append (ds->list_store, &iter);

	gtk_list_store_set (ds->list_store, &iter,
	                    DOWNLOAD_COL_DATA, dl,
	                    -1);

	download_store_unlock (ds);
	
	download_store_erase_excess (ds);
}

void download_store_add (DownloadStore* ds, Download* dl)
{
	Download* temp;

	temp = download_new ();
	download_assign (temp, dl);

	download_store_add_static (ds, temp);
}

void download_store_erase (DownloadStore* ds, GtkTreeIter* iter)
{
	Download* dl_data;
	gpointer  thread;

	download_store_lock (ds);

	gtk_tree_model_get ( (GtkTreeModel*)ds->list_store, iter,
	                    DOWNLOAD_COL_DATA, &dl_data,
	                    DOWNLOAD_COL_THREAD, &thread,
	                    -1);
	if (thread)
		download_thread_destroy (thread);

	download_unref (dl_data);
	gtk_list_store_remove (ds->list_store, iter);

	download_store_unlock (ds);
}

void download_store_erase_excess (DownloadStore* ds)
{
	guint n;

	n = gtk_tree_model_iter_n_children (GTK_TREE_MODEL (ds->list_store), NULL);

	if (ds->historical && n > ds->capacity_limit)
		download_store_erase_from_nth (ds, ds->capacity_limit);
}

void download_store_erase_from_nth (DownloadStore* ds, guint nth)
{
	GtkTreeIter iter, iter_next;
	gboolean    valid;

	if (ds->historical==FALSE)
		return;

	valid = gtk_tree_model_iter_nth_child (GTK_TREE_MODEL (ds->list_store),
	                                       &iter, NULL, nth);
    while (valid) {
    	iter_next = iter;
    	valid = gtk_tree_model_iter_next (GTK_TREE_MODEL (ds->list_store),
    	                                  &iter_next);
    	download_store_erase (ds, &iter);
    }
}

void download_store_move_to_store (DownloadStore* ds_src,
                                   DownloadStore* ds_dest,
                                   GtkTreeIter* iter_src)
{
	Download*        dl_data;
	DownloadThread*  dt;

	download_store_lock (ds_src);

	gtk_tree_model_get ((GtkTreeModel*)ds_src->list_store,
	                    iter_src,
	                    DOWNLOAD_COL_DATA, &dl_data,
	                    DOWNLOAD_COL_THREAD, &dt,
	                    -1);
    if (dt)
        download_thread_job_erased (dt);

	download_store_lock (ds_dest);

	download_store_add (ds_dest, dl_data);
	gtk_list_store_remove (ds_src->list_store, iter_src);

	download_store_unlock (ds_dest);

	download_store_unlock (ds_src);
}

gboolean download_store_find_waiting (DownloadStore* ds,
                                      GtkTreeIter*   iter)
{
	Download* dl_data;
	gboolean  return_val = FALSE;
	gboolean  valid;
	gpointer  thread;

//	download_store_lock (ds);

	valid = gtk_tree_model_get_iter_first (GTK_TREE_MODEL(ds->list_store),
	                                       iter);

	while (valid) {
		gtk_tree_model_get ( (GtkTreeModel*)ds->list_store, iter,
		                    DOWNLOAD_COL_DATA,   &dl_data,
		                    DOWNLOAD_COL_THREAD, &thread,
		                    -1);
		if (DOWNLOAD_SETTING (dl_data)->state == DOWNLOAD_STATE_WAITING &&
		    thread==NULL )
		{
			return_val = TRUE;
			break;
		}
		valid = gtk_tree_model_iter_next ( (GtkTreeModel*)ds->list_store,
		                                   iter);
	}

//	download_store_unlock (ds);

	return return_val;
}

void download_store_row_changed (DownloadStore* ds,
                                 GtkTreeIter* iter)
{
	GtkTreePath* path;

	path = gtk_tree_model_get_path ((GtkTreeModel*)ds->list_store,
	                                iter);
	gtk_tree_model_row_changed ((GtkTreeModel*)ds->list_store,
	                            path, iter);
	gtk_tree_path_free (path);
}

void download_store_set_thread(DownloadStore*  ds,
                               GtkTreeIter*    iter,
                               gpointer        ptr)
{
	DownloadThread*    thread = ptr;

	gtk_list_store_set (ds->list_store,
	                    iter,
	                    DOWNLOAD_COL_THREAD,
	                    thread,
	                    -1);
}

