#include "urlget.h"
#include "main_app.h"
#include "category_view.h"
#include "html_import_dialog.h"
#include "category_selection_dialog.h"
#include "xml_conf_parser.h"
#include "utility.h"

// utility ----------------------------------------------------------
gboolean file_selection_get_from_user (const gchar* window_title,
                                       const gchar* init_filename,
                                       GtkWindow* parent,
                                       gchar** filename)
{
	GtkFileSelection* selector;
	gboolean  return_val = FALSE;

	selector = (GtkFileSelection*)gtk_file_selection_new (window_title);

	if (init_filename)
		gtk_file_selection_complete (selector, init_filename);

	gtk_window_set_transient_for (GTK_WINDOW(selector), parent);

	if (gtk_dialog_run (GTK_DIALOG(selector))==GTK_RESPONSE_OK ) {
		*filename = g_strdup (gtk_file_selection_get_filename (selector));
		return_val = TRUE;
	}
	gtk_widget_destroy (GTK_WIDGET(selector));

	return return_val;
}

void show_file_error_message (GtkWindow* parent,
                              const gchar* format_string,
                              const gchar* filename)
{
	GtkWidget* message_dialog;

	message_dialog = gtk_message_dialog_new (parent,
	                                 GTK_DIALOG_DESTROY_WITH_PARENT,
	                                 GTK_MESSAGE_ERROR,
	                                 GTK_BUTTONS_CLOSE,
	                                 format_string, filename);
	gtk_window_set_transient_for (GTK_WINDOW(message_dialog), parent);
	gtk_dialog_run (GTK_DIALOG (message_dialog));
	gtk_widget_destroy (message_dialog);
}
// End of utility ===================================================


void main_app_import_html_auto (MainApp* app,
                                HtmlImport* importer,
                                Category*   category,
                                Download*   download_data)
{
	HtmlImportIter   iter;
	gboolean         valid;
	UrlPart          urlp;
	const gchar*     url_string;
	DownloadSetting* setting = DOWNLOAD_SETTING (download_data);

	valid = html_import_get_url_first (importer, &iter, &url_string);

	// clear string 
	download_setting_set_url (setting, NULL);
	download_setting_set_filename (setting, NULL);

	while (valid) {
		url_part (&urlp, url_string);
		setting->url = (gchar*)url_string;
		setting->filename = (urlp.name_beg) ? urlp.name_beg : "noname";

		download_store_add (&category->waiting_store, download_data);
		valid = html_import_get_url_next (importer, &iter, &url_string);
	}

	setting->filename = NULL;
	setting->url = NULL;
}

void main_app_on_import_html (MainApp* app)
{
	HtmlImport*       importer;
	HtmlImportDialog* import_dialog;
	gchar*    filename = NULL;
	gboolean  had_selected;
	gint      response;

	Category* selected_category;
	Download* download_data;

	had_selected = category_tree_view_get_selected (app->window.category_view,
	                                                &selected_category,
	                                                NULL, NULL);
	if ( had_selected == FALSE )
		return;

	had_selected = file_selection_get_from_user (_("Select HTML file to import"),
	                                             g_get_home_dir (),
	                                             app->window.self,
	                                             &filename);
	if ( had_selected == FALSE )
		return;

	if (g_file_test (filename, G_FILE_TEST_EXISTS) == FALSE) {
		show_file_error_message (app->window.self,
		                         _("%s isn't a valid file."),
		                         filename);

		g_free (filename);
		return;
	}

	importer = html_import_new ();
	html_import_add_grabbed_element (importer, "A", "HREF", _("Link <A>"));
	html_import_add_grabbed_element (importer, "IMG", "SRC", _("Image <IMG>"));
	html_import_read_file (importer, filename);
	g_free (filename);

	import_dialog = html_import_dialog_new (importer, app->window.self);
	response = gtk_dialog_run (import_dialog->self);
	html_import_dialog_destroy (import_dialog);

	if (response==GTK_RESPONSE_OK) {
		download_data = download_new ();

		main_app_get_download_from_user (app,
		                                 DOWNLOAD_SETTING (download_data),
		                                 &selected_category,
		                                 TRUE);
		main_app_import_html_auto (app, importer,
		                           selected_category,
		                           download_data);

		download_unref (download_data);
	}

	html_import_destroy (importer);
}

void main_app_on_save (MainApp* app)
{
	FILE* file;

	make_path_exist (urlget_get_conf_dir ());

	// write setting
	file = fopen (urlget_get_conf_filename (), "w");
	fputs ("<?xml version=\"1.0\"?>\n", file);
	global_setting_write_xml (app->setting, file, 0);
	fclose (file);

	// write category data
	file = fopen (urlget_get_data_filename (), "w");
	fputs ("<?xml version=\"1.0\"?>\n", file);

	category_store_write_xml (&app->category_store, file, 0);
	fclose (file);
}

void main_app_on_save_as (MainApp* app)
{
	FILE*  file;
	gboolean  had_ok;
	gchar*    filename = NULL;

	had_ok = file_selection_get_from_user (_("Save category data as..."),
	                                       g_get_home_dir (),
	                                       app->window.self,
	                                       &filename);

	if (had_ok) {
		puts ("write data start ----");
		// write category data
		file = fopen (filename, "w");
		fputs ("<?xml version=\"1.0\"?>\n", file);
		category_store_write_xml (&app->category_store, file, 0);
		fclose (file);
		puts ("write data end ----");
	}

	g_free (filename);
}

void main_app_on_load (MainApp* app)
{
	gboolean  had_ok;
	gchar*    filename = NULL;

	had_ok = file_selection_get_from_user (_("Load category data"),
	                                       g_get_home_dir (),
	                                       app->window.self,
	                                       &filename);

	if (had_ok) {
		had_ok = main_app_read_data_file (app, filename);

		if (had_ok==FALSE) {
			show_file_error_message (app->window.self,
			                         _("%s load error."),
			                         filename);
		}
	}

	g_free (filename);
}

void main_app_on_export_url (MainApp* app)
{
	FILE*     file;
	gboolean  had_ok;
	gchar*    filename = NULL;
	gint      response;

	CategorySelectionDialog* csd;
	DownloadStore*           download_store;
	DownloadSetting*         setting;
	GtkTreeIter iter;

	csd = category_selection_dialog_new (_("Select one to export URLs"),
	                                     &app->category_store,
	                                     app->window.self);
	response = gtk_dialog_run (csd->self);
	category_tree_view_get_selected (csd->category_view, NULL,
	                                 NULL, &download_store);
	category_selection_dialog_destroy (csd);

	if (response != GTK_RESPONSE_OK)
		return;

	had_ok = file_selection_get_from_user (_("Export URLs"),
	                                       g_get_home_dir (),
	                                       app->window.self,
	                                       &filename);
	if (had_ok==FALSE)
		return;

	// write data to file
	file = fopen (filename, "w");
	g_free (filename);

	download_store_lock (download_store);

	had_ok = gtk_tree_model_get_iter_first (GTK_TREE_MODEL (download_store->list_store),
	                                        &iter);

	while (had_ok) {
		gtk_tree_model_get (GTK_TREE_MODEL (download_store->list_store),
		                    &iter,
		                    DOWNLOAD_COL_DATA,
		                    &setting,
		                    -1);

		fputs (setting->url, file);
		fputs ("\n", file);

		had_ok = gtk_tree_model_iter_next (GTK_TREE_MODEL (download_store->list_store),
		                                   &iter);
	}

	download_store_unlock (download_store);

	fclose (file);
}

gboolean main_app_read_setting_file (MainApp* app, const gchar* filename)
{
	GError* error = NULL;
	GlobalSetting* gs = NULL;
	XmlConfParser* xcp;

	// read setting file
	if (g_file_test (filename, G_FILE_TEST_EXISTS)) {
		xcp = xml_conf_parser_new ();
		gs = xml_conf_parser_read_setting (xcp,
		                                   filename,
		                                   &error);
		xml_conf_parser_destroy (xcp);

		if (error)
			g_error_free (error);

		if (gs) {
			if (app->setting)
				global_setting_destroy (app->setting);
			app->setting = gs;
			return TRUE;
		}
	}

	return FALSE;
}

gboolean main_app_read_data_file (MainApp* app, const gchar* filename)
{
	XmlConfParser* xcp;
	GList    *list, *node;
	GError*   error = NULL;

	if (g_file_test (filename, G_FILE_TEST_IS_REGULAR)) {
		xcp = xml_conf_parser_new ();
		list = xml_conf_parser_read_category (xcp,
		                                      filename,
		                                      &error);
		xml_conf_parser_destroy (xcp);

		if (error)
			g_error_free (error);

		if (list==NULL)
			return FALSE;

		for (node=list; node; node=node->next) {
			category_apply_setting ((Category*)node->data);
			category_store_append (&app->category_store,
			                       (Category*)node->data);
			main_app_connect_category_signal (app, (Category*)node->data);
		}
		g_list_free (list);

		return TRUE;
	}

	return FALSE;
}

