#include "xml_conf_parser.h"


void xml_conf_parser_start_element (GMarkupParseContext* context,
                                    const gchar*         element_name,
                                    const gchar**        attr_names,
                                    const gchar**        attr_values,
                                    XmlConfParser*       xcp,
                                    GError**             error);

void xml_conf_parser_end_element (GMarkupParseContext* context,
                                  const gchar*         element_name,
                                  XmlConfParser*       xcp,
                                  GError**             error);

void xml_conf_parser_text (GMarkupParseContext* context,
                           const gchar*         text,
                           gsize                text_len,
                           XmlConfParser*       xcp,
                           GError**             error);


static const gpointer empty_pointer = NULL;

XmlConfParser* xml_conf_parser_new ()
{
	XmlConfParser* xcp = g_malloc (sizeof (XmlConfParser));

	xml_conf_parser_init (xcp);

	return xcp;
}

void xml_conf_parser_destroy (XmlConfParser* xcp)
{
	xml_conf_parser_clear (xcp);
	g_free (xcp);
}

void xml_conf_parser_init (XmlConfParser* xcp)
{
	xcp->parser.start_element = xml_conf_parser_start_element;
	xcp->parser.end_element = xml_conf_parser_end_element;
	xcp->parser.text  = xml_conf_parser_text;
	xcp->parser.passthrough = NULL;
	xcp->parser.error = NULL;

	xcp->parser_context = g_markup_parse_context_new (&xcp->parser, 0,
	                                                  xcp, NULL);
	xcp->parser_array = g_ptr_array_sized_new (16);
}

void xml_conf_parser_clear (XmlConfParser* xcp)
{
	g_markup_parse_context_free (xcp->parser_context);
	g_ptr_array_free (xcp->parser_array, TRUE);
}

void xml_conf_parser_stack_push (XmlConfParser*       xcp,
                                 xml_conf_parser_func func,
                                 gpointer             data)
{
	gpointer* pdata;
	int len = xcp->parser_array->len;

	g_ptr_array_set_size (xcp->parser_array, len+2);

	pdata = xcp->parser_array->pdata;
	pdata[len]   = (gpointer)func;
	pdata[len+1] = data;
}

void xml_conf_parser_stack_pop (XmlConfParser*  xcp)
{
	if (xcp->parser_array->len >= 2)
		g_ptr_array_set_size (xcp->parser_array, xcp->parser_array->len -2);
}

void xml_conf_parser_stack_call (XmlConfParser*  xcp)
{
	GPtrArray*   parser_array = xcp->parser_array;
	xml_conf_parser_func   parse_func;

	if (parser_array->len) {
		parse_func = g_ptr_array_index (parser_array, parser_array->len-2);
		parse_func (xcp, g_ptr_array_index (parser_array, parser_array->len-1));
	}
}

gboolean xml_conf_parser_read_file (XmlConfParser* xcp,
                                    const gchar* filename,
                                    GError** error)
{
	FILE*    file;
	guint8*  buffer;
	guint    size;
	gboolean parse_ok = TRUE;

	file = fopen (filename, "r");
	if (file==NULL)
		return FALSE;

	buffer = g_malloc (4096);

	while ( (size=fread (buffer, 1, 4096, file)) && parse_ok) {
		parse_ok = g_markup_parse_context_parse (xcp->parser_context,
		                                         buffer, size, error);
	}

	g_markup_parse_context_end_parse (xcp->parser_context, NULL);
	g_free (buffer);
	fclose (file);

	return parse_ok;
}

void xml_conf_parser_start_element (GMarkupParseContext* context,
                                    const gchar*         element_name,
                                    const gchar**        attr_names,
                                    const gchar**        attr_values,
                                    XmlConfParser*       xcp,
                                    GError**             error)
{
	xcp->element_name = element_name;
	xcp->attr_names   = attr_names;
	xcp->attr_values  = attr_values;
	xcp->text         = NULL;
	xcp->text_len     = 0;
	xcp->closing_tag  = FALSE;

	xml_conf_parser_stack_call (xcp);
}

void xml_conf_parser_end_element (GMarkupParseContext* context,
                                  const gchar*         element_name,
                                  XmlConfParser*       xcp,
                                  GError**             error)
{
	/*
	xcp->element_name = element_name;
	xcp->attr_names   = (const gchar**)&empty_pointer;
	xcp->attr_values  = (const gchar**)&empty_pointer;
	xcp->text         = NULL;
	xcp->text_len     = 0;
	xcp->closing_tag  = TRUE;

	xml_conf_parser_stack_call (xcp);
	*/
	xml_conf_parser_stack_pop (xcp);
}

void xml_conf_parser_text (GMarkupParseContext* context,
                           const gchar*         text,
                           gsize                text_len,
                           XmlConfParser*       xcp,
                           GError**             error)
{
	xcp->element_name = NULL;
	xcp->attr_names   = (const gchar**)&empty_pointer;
	xcp->attr_values  = (const gchar**)&empty_pointer;
	xcp->text         = text;
	xcp->text_len     = text_len;

	xml_conf_parser_stack_call (xcp);
}


